@@ -28,7 +28,6 @@
 #include "NstCore.hpp"
 #include "NstAssert.hpp"
 #include "NstFpuPrecision.hpp"
-#include "api/NstApiVideo.hpp"
 #include "NstVideoRenderer.hpp"
 #include "NstVideoFilterNone.hpp"
 
@@ -471,6 +470,7 @@ namespace Nes
 			void Renderer::Palette::Update(int brightness,int saturation,int contrast,int hue)
 			{
 				FpuPrecision precision;
+				(void)precision;
 				(*this.*(type == PALETTE_YUV ? &Palette::Generate : &Palette::Build))( brightness, saturation, contrast, hue );
 			}
 
