@@ -17,6 +17,8 @@ public:
 
     void rehash();
 
+    int init();
+
     void unload_game();
     void load_game(const char *filename, std::vector<uint8_t>& game);
     bool is_loaded();
@@ -66,6 +68,7 @@ private:
     std::string gamepath;
     std::string gamename;
     std::string gamefname;
+    std::string gamemd5;
 
     std::vector<jg_setting_t*> settings;
 };
